# -*- coding: utf-8 -*-
"""Python lager brewed by a loguru"""
from lager import logging
from lager.__about__ import __version__
from lager.const import LAGER_PORT, LOGURU_DEFAULT_FMT, TORNADO_FMT
from lager.core import (
    LAGER,
    LN,
    LOG,
    _change_activation,
    _core,
    _find_iter,
    _log,
    _options,
    add,
    bind,
    catch,
    complete,
    configure,
    contextualize,
    critical,
    debug,
    disable,
    enable,
    error,
    exception,
    flog,
    handlers,
    info,
    lager,
    level,
    ln,
    log,
    logger,
    loglevel,
    opt,
    parse,
    patch,
    remove,
    reset,
    start,
    stop,
    success,
    trace,
    warning,
)

__all__ = (
    "__version__",
    "LAGER_PORT",
    "LOGURU_DEFAULT_FMT",
    "TORNADO_FMT",
    "loglevel",
    "handlers",
    "flog",
    "logger",
    "logging",
    "lager",
    "LAGER",
    "LOG",
    "log",
    "ln",
    "LN",
    "reset",
    #############
    ## HOISTED ##
    #############
    "_change_activation",
    "_core",
    "_find_iter",
    "_log",
    "_options",
    "add",
    "bind",
    "catch",
    "complete",
    "configure",
    "contextualize",
    "critical",
    "debug",
    "disable",
    "enable",
    "error",
    "exception",
    "info",
    "level",
    "opt",
    "parse",
    "patch",
    "remove",
    "start",
    "stop",
    "success",
    "trace",
    "warning",
)
