# -*- coding: utf-8 -*-
"""Package metadata/info"""

__all__ = ("__title__", "__description__", "__pkgroot__", "__version__")
__title__ = "lager"
__description__ = "EZ-PZ logging based on loguru"
__pkgroot__ = __file__.replace("__about__.py", "").rstrip("/\\")
__version__ = "0.17.2"
